@@ -17,13 +17,19 @@ except ImportError:
 use_mpi4py = False
 MPI = None
 
-# get the path to the MDI Library
+# get the name of the MDI Library
 mdi_name_path = os.path.join( dir_path, "mdi_name" )
 mdi_name_file = open( mdi_name_path, "r" )
 mdi_name = mdi_name_file.read()
-mdi_path = os.path.join( dir_path, mdi_name )
 mdi_name_file.close()
 
+# check if mdi_name is a full path to the MDI Library
+if os.path.exists(mdi_name):
+    mdi_path = mdi_name
+else:
+    # the mdi_name file includes only the name of the library
+    mdi_path = os.path.join( dir_path, mdi_name )
+
 # load the MDI Library
 try:
     mdi = ctypes.CDLL(mdi_path, ctypes.RTLD_GLOBAL)
@@ -676,6 +682,20 @@ def MDI_MPI_set_world_comm(new_comm):
     global intra_code_comm
     intra_code_comm = new_comm
 
+# MDI_Check_Command_Exists
+mdi.MDI_Check_for_communicator.argtypes = [ctypes.POINTER(ctypes.c_int)]
+mdi.MDI_Check_for_communicator.restype = ctypes.c_int
+def MDI_Check_for_communicator():
+    arg_size = ctypes.sizeof(ctypes.c_int)
+    flag = (ctypes.c_int*arg_size)()
+
+    ret = mdi.MDI_Check_for_communicator(flag)
+    if ret != 0:
+        raise Exception("MDI Error: MDI_Check_Command_Exists failed")
+    flag_cast = ctypes.cast(flag, ctypes.POINTER(ctypes.c_int)).contents
+
+    return flag_cast.value
+
 # MDI_Accept_Communicator
 mdi.MDI_Accept_Communicator.argtypes = [ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Accept_Communicator.restype = ctypes.c_int
