@@ -37,6 +37,7 @@ int enable_plug_support( int code_id ) {
     return ret;
   }
   this_method->on_selection = plug_on_selection;
+  this_method->on_check_communicator = plug_on_check_communicator;
   this_method->on_accept_communicator = plug_on_accept_communicator;
   this_method->on_send_command = plug_on_send_command;
   this_method->after_send_command = plug_after_send_command;
@@ -68,6 +69,36 @@ int plug_on_selection() {
 
 
 
+/*! \brief Callback when the PLUG method must check whether a new communicator exists */
+int plug_on_check_communicator(int* flag) {
+  int ret;
+
+  ret = mdi_debug("[MDI:plug_on_accept_communicator] Start\n");
+  if ( ret != 0 ) {
+    mdi_error("Error in plug_on_accept_communicator: mdi_debug failed");
+    return ret;
+  }
+
+  code* this_code;
+  ret = get_current_code(&this_code);
+  if ( ret != 0 ) {
+    mdi_error("Error in plug_on_accept_communicator: get_current_code failed");
+    return ret;
+  }
+
+  // If MDI hasn't returned some connections, do that now
+  if ( this_code->returned_comms < this_code->next_comm - 1 ) {
+    *flag = 1;
+  }
+  else {
+    *flag = 0;
+  }
+
+  return 0;
+}
+
+
+
 /*! \brief Callback when the PLUG method must accept a communicator */
 int plug_on_accept_communicator() {
   int ret;
@@ -82,7 +113,7 @@ int plug_on_accept_communicator() {
   ret = get_current_code(&this_code);
   if ( ret != 0 ) {
     mdi_error("Error in plug_on_accept_communicator: get_current_code failed");
-    return 1;
+    return ret;
   }
 
   // Give the library method an opportunity to update the current code
@@ -322,6 +353,8 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
   int exists_flag;
   ret = file_exists( plugin_path, &exists_flag );
   if ( ret != 0 ) {
+    free( plugin_path );
+    free( plugin_init_name );
     mdi_error("Error in library_load_init: file_exists failed");
     return ret;
   }
@@ -329,6 +362,8 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
 
     ret = mdi_debug("[MDI:library_load_init] Attempting to load a python plugin\n");
     if ( ret != 0 ) {
+      free( plugin_path );
+      free( plugin_init_name );
       mdi_error("Error in library_load_init: mdi_debug failed");
       return ret;
     }
@@ -337,6 +372,8 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
     libd->shared_state->engine_language = MDI_LANGUAGE_PYTHON;
     ret = python_plugin_init( plugin_name, plugin_path, mpi_comm_ptr, libd->shared_state, mode );
     if ( ret != 0 ) {
+      free( plugin_path );
+      free( plugin_init_name );
       mdi_error("Error in python_plugin_init");
       return -1;
     }
@@ -351,6 +388,7 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
   if ( ! libd->plugin_handle ) {
     // Unable to find the plugin library
     free( plugin_path );
+    free( plugin_init_name );
     mdi_error("Unable to open MDI plugin");
     return -1;
   }
@@ -358,11 +396,25 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
   // Load a plugin's initialization function
   libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) GetProcAddress( libd->plugin_handle, plugin_init_name );
   if ( ! libd->plugin_init ) {
-    free( plugin_path );
-    free( plugin_init_name );
-    FreeLibrary( libd->plugin_handle );
-    mdi_error("Unable to load MDI plugin init function");
-    return -1;
+    // If this is a call to MDI_Launch_plugin, also try MDI_Plugin_launch_<plugin_name>
+    if ( mode == 0 ) {
+      snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_launch_%s", plugin_name);
+      libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) GetProcAddress( libd->plugin_handle, plugin_init_name );
+      if ( ! libd->plugin_init ) {
+        free( plugin_path );
+        free( plugin_init_name );
+        FreeLibrary( libd->plugin_handle );
+        mdi_error("Unable to load MDI plugin launch function");
+        return -1;
+      }
+    }
+    else {
+      free( plugin_path );
+      free( plugin_init_name );
+      FreeLibrary( libd->plugin_handle );
+      mdi_error("Unable to load MDI plugin open function");
+      return -1;
+    }
   }
 
 #else
@@ -387,11 +439,24 @@ int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
   // Load a plugin's initialization function
   libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) dlsym(libd->plugin_handle, plugin_init_name);
   if ( ! libd->plugin_init ) {
-    free( plugin_path );
-    free( plugin_init_name );
-    dlclose( libd->plugin_handle );
-    mdi_error("Unable to load MDI plugin init function");
-    return -1;
+    if ( mode == 0 ) {
+      snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_launch_%s", plugin_name);
+      libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) dlsym(libd->plugin_handle, plugin_init_name);
+      if ( ! libd->plugin_init ) {
+        free( plugin_path );
+        free( plugin_init_name );
+        dlclose( libd->plugin_handle );
+        mdi_error("Unable to load MDI plugin launch function");
+        return -1;
+      }
+    }
+    else {
+      free( plugin_path );
+      free( plugin_init_name );
+      dlclose( libd->plugin_handle );
+      mdi_error("Unable to load MDI plugin open function");
+      return -1;
+    }
   }
 #endif
 
